@@ -1,4 +1,4 @@
-{-# LANGUAGE DeriveDataTypeable, OverloadedStrings #-}
+{-# LANGUAGE CPP, DeriveDataTypeable, OverloadedStrings #-}
 
 -- | This module re-exports the @Github.Data.Definitions@ module, adding
 -- instances of @FromJSON@ to it. If you wish to use the data without the
@@ -6,17 +6,21 @@
 
 module Github.Data (module Github.Data.Definitions) where
 
-import Data.Time
 import Control.Applicative
 import Control.Monad
 import qualified Data.Text as T
 import Data.Aeson.Types
-import Data.Monoid
-import System.Locale (defaultTimeLocale)
 import qualified Data.Vector as V
 import qualified Data.HashMap.Lazy as Map
 import Data.Hashable (Hashable)
 
+#if MIN_VERSION_base(4,8,0)
+import Data.Time
+#else
+import Data.Time
+import System.Locale (defaultTimeLocale)
+#endif
+
 import Github.Data.Definitions
 
 instance FromJSON GithubDate where
@@ -49,7 +53,7 @@ instance FromJSON GitTree where
   parseJSON (Object o) =
     GitTree <$> o .: "type"
          <*> o .: "sha"
-         <*> o .: "url"
+         <*> o .:? "url"
          <*> o .:? "size"
          <*> o .: "path"
          <*> o .: "mode"
@@ -113,12 +117,12 @@ instance FromJSON Comment where
     Comment <$> o .:? "position"
             <*> o .:? "line"
             <*> o .: "body"
-            <*> o .: "commit_id"
+            <*> o .:? "commit_id"
             <*> o .: "updated_at"
             <*> o .:? "html_url"
             <*> o .: "url"
             <*> o .: "created_at"
-            <*> o .: "path"
+            <*> o .:? "path"
             <*> o .: "user"
             <*> o .: "id"
   parseJSON _          = fail "Could not build a Comment"
@@ -190,6 +194,9 @@ instance FromJSON Blob where
          <*> o .: "size"
   parseJSON _          = fail "Could not build a Blob"
 
+instance ToJSON NewGitReference where
+  toJSON (NewGitReference r s) = object [ "ref" .= r, "sha" .= s  ]
+
 instance FromJSON GitReference where
   parseJSON (Object o) =
     GitReference <$> o .: "object"
@@ -208,6 +215,7 @@ instance FromJSON Issue where
   parseJSON (Object o) =
     Issue <$> o .:? "closed_at"
           <*> o .: "updated_at"
+          <*> o .: "events_url"
           <*> o .: "html_url"
           <*> o .:? "closed_by"
           <*> o .: "labels"
@@ -303,6 +311,16 @@ instance FromJSON EventType where
   parseJSON (String "mentioned") = pure Mentioned
   parseJSON (String "assigned") = pure Assigned
   parseJSON (String "unsubscribed") = pure Unsubscribed
+  parseJSON (String "unassigned") = pure ActorUnassigned
+  parseJSON (String "labeled") = pure Labeled
+  parseJSON (String "unlabeled") = pure Unlabeled
+  parseJSON (String "milestoned") = pure Milestoned
+  parseJSON (String "demilestoned") = pure Demilestoned
+  parseJSON (String "renamed") = pure Renamed
+  parseJSON (String "locked") = pure Locked
+  parseJSON (String "unlocked") = pure Unlocked
+  parseJSON (String "head_ref_deleted") = pure HeadRefDeleted
+  parseJSON (String "head_ref_restored") = pure HeadRefRestored
   parseJSON _ = fail "Could not build an EventType"
 
 instance FromJSON SimpleOrganization where
@@ -354,6 +372,22 @@ instance FromJSON PullRequest where
         <*> o .: "id"
   parseJSON _ = fail "Could not build a PullRequest"
 
+instance ToJSON EditPullRequestState where
+  toJSON (EditPullRequestStateOpen) = String "open"
+  toJSON (EditPullRequestStateClosed) = String "closed"
+
+instance ToJSON EditPullRequest where
+  toJSON (EditPullRequest t b s) =
+    object $ filter notNull [ "title" .= t, "body" .= b, "state" .= s ]
+    where notNull (_, Null) = False
+          notNull (_, _) = True
+
+instance ToJSON CreatePullRequest where
+  toJSON (CreatePullRequest t b headPR basePR) =
+    object [ "title" .= t, "body" .= b, "head" .= headPR, "base" .= basePR ]
+  toJSON (CreatePullRequestIssue issueNum headPR basePR) =
+    object [ "issue" .= issueNum, "head" .= headPR, "base" .= basePR]
+
 instance FromJSON DetailedPullRequest where
   parseJSON (Object o) =
       DetailedPullRequest
@@ -383,7 +417,7 @@ instance FromJSON DetailedPullRequest where
         <*> o .: "base"
         <*> o .: "commits"
         <*> o .: "merged"
-        <*> o .: "mergeable"
+        <*> o .:? "mergeable"
   parseJSON _ = fail "Could not build a DetailedPullRequest"
 
 instance FromJSON PullRequestLinks where
@@ -395,10 +429,86 @@ instance FromJSON PullRequestLinks where
   parseJSON _ = fail "Could not build a PullRequestLinks"
 
 instance FromJSON PullRequestCommit where
-  parseJSON (Object _) =
-    return PullRequestCommit
+  parseJSON (Object o) =
+    PullRequestCommit <$> o .: "label"
+                      <*> o .: "ref"
+                      <*> o .: "sha"
+                      <*> o .: "user"
+                      <*> o .: "repo"
   parseJSON _ = fail "Could not build a PullRequestCommit"
 
+instance FromJSON PullRequestEvent where
+  parseJSON (Object o) =
+    PullRequestEvent <$> o .: "action"
+                     <*> o .: "number"
+                     <*> o .: "pull_request"
+                     <*> o .: "repository"
+                     <*> o .: "sender"
+  parseJSON _ = fail "Could not build a PullRequestEvent"
+
+instance FromJSON PullRequestEventType where
+  parseJSON (String "opened") = pure PullRequestOpened
+  parseJSON (String "closed") = pure PullRequestClosed
+  parseJSON (String "synchronize") = pure PullRequestSynchronized
+  parseJSON (String "reopened") = pure PullRequestReopened
+  parseJSON (String "assigned") = pure PullRequestAssigned
+  parseJSON (String "unassigned") = pure PullRequestUnassigned
+  parseJSON (String "labeled") = pure PullRequestLabeled
+  parseJSON (String "unlabeled") = pure PullRequestUnlabeled
+  parseJSON _ = fail "Could not build a PullRequestEventType"
+
+instance FromJSON RepoWebhookEvent where
+  parseJSON (String "*") = pure WebhookWildcardEvent
+  parseJSON (String "commit_comment") = pure WebhookCommitCommentEvent
+  parseJSON (String "create") = pure WebhookCreateEvent
+  parseJSON (String "delete") = pure WebhookDeleteEvent
+  parseJSON (String "deployment") = pure WebhookDeploymentEvent
+  parseJSON (String "deployment_status") = pure WebhookDeploymentStatusEvent
+  parseJSON (String "fork") = pure WebhookForkEvent
+  parseJSON (String "gollum") = pure WebhookGollumEvent
+  parseJSON (String "issue_comment") = pure WebhookIssueCommentEvent
+  parseJSON (String "issues") = pure WebhookIssuesEvent
+  parseJSON (String "member") = pure WebhookMemberEvent
+  parseJSON (String "page_build") = pure WebhookPageBuildEvent
+  parseJSON (String "public") = pure WebhookPublicEvent
+  parseJSON (String "pull_request_review_comment") = pure WebhookPullRequestReviewCommentEvent
+  parseJSON (String "pull_request") = pure WebhookPullRequestEvent
+  parseJSON (String "push") = pure WebhookPushEvent
+  parseJSON (String "release") = pure WebhookReleaseEvent
+  parseJSON (String "status") = pure WebhookStatusEvent
+  parseJSON (String "team_add") = pure WebhookTeamAddEvent
+  parseJSON (String "watch") = pure WebhookWatchEvent
+  parseJSON _ = fail "Could not build a Webhook event"
+
+instance ToJSON RepoWebhookEvent where
+  toJSON (WebhookWildcardEvent) = String "*"
+  toJSON (WebhookCommitCommentEvent) = String "commit_comment"
+  toJSON (WebhookCreateEvent) = String "create"
+  toJSON (WebhookDeleteEvent) = String "delete"
+  toJSON (WebhookDeploymentEvent) = String "deployment"
+  toJSON (WebhookDeploymentStatusEvent) = String "deployment_status"
+  toJSON (WebhookForkEvent) = String "fork"
+  toJSON (WebhookGollumEvent) = String "gollum"
+  toJSON (WebhookIssueCommentEvent) = String "issue_comment"
+  toJSON (WebhookIssuesEvent) = String "issues"
+  toJSON (WebhookMemberEvent) = String "member"
+  toJSON (WebhookPageBuildEvent) = String "page_build"
+  toJSON (WebhookPublicEvent) = String "public"
+  toJSON (WebhookPullRequestReviewCommentEvent) = String "pull_request_review_comment"
+  toJSON (WebhookPullRequestEvent) = String "pull_request"
+  toJSON (WebhookPushEvent) = String "push"
+  toJSON (WebhookReleaseEvent) = String "release"
+  toJSON (WebhookStatusEvent) = String "status"
+  toJSON (WebhookTeamAddEvent) = String "team_add"
+  toJSON (WebhookWatchEvent) = String "watch"
+
+instance FromJSON PingEvent where
+  parseJSON (Object o) =
+    PingEvent <$> o .: "zen"
+              <*> o .: "hook"
+              <*> o .: "hook_id"
+  parseJSON _ = fail "Could not build a PingEvent"
+
 instance FromJSON SearchReposResult where
   parseJSON (Object o) =
     SearchReposResult <$> o .: "total_count"
@@ -407,35 +517,53 @@ instance FromJSON SearchReposResult where
 
 instance FromJSON Repo where
   parseJSON (Object o) =
-    Repo <$> o .: "ssh_url"
+    Repo <$> o .:? "ssh_url"
          <*> o .: "description"
-         <*> o .: "created_at"
+         <*> o .:? "created_at"
          <*> o .: "html_url"
-         <*> o .: "svn_url"
-         <*> o .: "forks"
+         <*> o .:? "svn_url"
+         <*> o .:? "forks"
          <*> o .:? "homepage"
          <*> o .: "fork"
-         <*> o .: "git_url"
+         <*> o .:? "git_url"
          <*> o .: "private"
-         <*> o .: "clone_url"
-         <*> o .: "size"
-         <*> o .: "updated_at"
-         <*> o .: "watchers"
+         <*> o .:? "clone_url"
+         <*> o .:? "size"
+         <*> o .:? "updated_at"
+         <*> o .:? "watchers"
          <*> o .: "owner"
          <*> o .: "name"
-         <*> o .: "language"
+         <*> o .:? "language"
          <*> o .:? "master_branch"
-         <*> o .: "pushed_at"
+         <*> o .:? "pushed_at"
          <*> o .: "id"
          <*> o .: "url"
-         <*> o .: "open_issues"
+         <*> o .:? "open_issues"
          <*> o .:? "has_wiki"
          <*> o .:? "has_issues"
          <*> o .:? "has_downloads"
 	 <*> o .:? "parent"
 	 <*> o .:? "source"
+         <*> o .: "hooks_url"
   parseJSON _ = fail "Could not build a Repo"
 
+instance FromJSON SearchCodeResult where
+  parseJSON (Object o) =
+    SearchCodeResult <$> o .: "total_count"
+                     <*> o .:< "items"
+  parseJSON _ = fail "Could not build a SearchCodeResult"
+
+instance FromJSON Code where
+  parseJSON (Object o ) =
+    Code <$> o .: "name"
+         <*> o .: "path"
+         <*> o .: "sha"
+         <*> o .: "url"
+         <*> o .: "git_url"
+         <*> o .: "html_url"
+         <*> o .: "repository"
+  parseJSON _ = fail "Could not build a Code"
+
 instance FromJSON RepoRef where
   parseJSON (Object o) =
     RepoRef <$> o .: "owner"
@@ -505,7 +633,7 @@ instance FromJSON DetailedOwner where
                    <*> o .: "avatar_url"
                    <*> o .: "followers"
                    <*> o .: "following"
-                   <*> o .: "hireable"
+                   <*> o .:? "hireable"
                    <*> o .: "gravatar_id"
                    <*> o .:? "blog"
                    <*> o .:? "bio"
@@ -513,13 +641,52 @@ instance FromJSON DetailedOwner where
                    <*> o .:? "name"
                    <*> o .:? "location"
                    <*> o .:? "company"
-                   <*> o .: "email"
+                   <*> o .:? "email"
                    <*> o .: "url"
                    <*> o .: "id"
                    <*> o .: "html_url"
                    <*> o .: "login"
   parseJSON _ = fail "Could not build a DetailedOwner"
 
+instance FromJSON RepoWebhook where
+  parseJSON (Object o) =
+    RepoWebhook <$> o .: "url"
+                <*> o .: "test_url"
+                <*> o .: "id"
+                <*> o .: "name"
+                <*> o .: "active"
+                <*> o .: "events"
+                <*> o .: "config"
+                <*> o .: "last_response"
+                <*> o .: "updated_at"
+                <*> o .: "created_at"
+  parseJSON _          = fail "Could not build a RepoWebhook"
+
+instance FromJSON RepoWebhookResponse where
+  parseJSON (Object o) =
+    RepoWebhookResponse <$> o .: "code"
+                        <*> o .: "status"
+                        <*> o .: "message"
+  parseJSON _          = fail "Could not build a RepoWebhookResponse"
+
+instance FromJSON Content where
+  parseJSON o@(Object _) = ContentFile <$> parseJSON o
+  parseJSON (Array os) = ContentDirectory <$> (mapM parseJSON $ V.toList os)
+  parseJSON _ = fail "Could not build a Content"
+
+instance FromJSON ContentData where
+  parseJSON (Object o) =
+    ContentData <$> o .: "type"
+                <*> o .: "encoding"
+                <*> o .: "size"
+                <*> o .: "name"
+                <*> o .: "path"
+                <*> o .: "content"
+                <*> o .: "sha"
+                <*> o .: "url"
+                <*> o .: "git_url"
+                <*> o .: "html_url"
+  parseJSON _ = fail "Could not build a ContentData"
 
 -- | A slightly more generic version of Aeson's @(.:?)@, using `mzero' instead
 -- of `Nothing'.
@@ -540,6 +707,7 @@ obj <.:> [key] = obj .: key
 obj <.:> (key:keys) =
   let (Object nextObj) = findWithDefault (Object Map.empty) key obj in
       nextObj <.:> keys
+_ <.:> [] = fail "must have a pair"
 
 -- | Produce the value for the given key, maybe.
 at :: Object -> T.Text -> Maybe Value
