@@ -5,6 +5,7 @@ module Github.Data.Definitions where
 import Data.Time
 import Data.Data
 import qualified Control.Exception as E
+import qualified Data.Map as M
 
 -- | Errors have been tagged according to their source, so you can more easily
 -- dispatch and handle them.
@@ -39,7 +40,8 @@ data Tree = Tree {
 data GitTree = GitTree {
   gitTreeType :: String
   ,gitTreeSha :: String
-  ,gitTreeUrl :: String
+  -- Can be empty for submodule
+  ,gitTreeUrl :: Maybe String
   ,gitTreeSize :: Maybe Int
   ,gitTreePath :: String
   ,gitTreeMode :: String
@@ -97,11 +99,11 @@ data Comment = Comment {
    commentPosition :: Maybe Int
   ,commentLine :: Maybe Int
   ,commentBody :: String
-  ,commentCommitId :: String
+  ,commentCommitId :: Maybe String
   ,commentUpdatedAt :: UTCTime
   ,commentHtmlUrl :: Maybe String
   ,commentUrl :: String
-  ,commentCreatedAt :: UTCTime
+  ,commentCreatedAt :: Maybe UTCTime
   ,commentPath :: Maybe String
   ,commentUser :: GithubOwner
   ,commentId :: Int
@@ -171,6 +173,11 @@ data Blob = Blob {
   ,blobSize :: Int
 } deriving (Show, Data, Typeable, Eq, Ord)
 
+data NewGitReference = NewGitReference {
+   newGitReferenceRef :: String
+  ,newGitReferenceSha :: String
+} deriving (Show, Data, Typeable, Eq, Ord)
+
 data GitReference = GitReference {
    gitReferenceObject :: GitObject
   ,gitReferenceUrl :: String
@@ -186,6 +193,7 @@ data GitObject = GitObject {
 data Issue = Issue {
    issueClosedAt :: Maybe GithubDate
   ,issueUpdatedAt :: GithubDate
+  ,issueEventsUrl :: String
   ,issueHtmlUrl :: Maybe String
   ,issueClosedBy :: Maybe GithubOwner
   ,issueLabels :: [IssueLabel]
@@ -265,6 +273,16 @@ data EventType =
   | Assigned      -- ^ The issue was assigned to the actor.
   | Closed        -- ^ The issue was closed by the actor. When the commit_id is present, it identifies the commit that closed the issue using “closes / fixes #NN” syntax. 
   | Reopened      -- ^ The issue was reopened by the actor.
+  | ActorUnassigned    -- ^ The issue was unassigned to the actor
+  | Labeled       -- ^ A label was added to the issue.
+  | Unlabeled     -- ^ A label was removed from the issue.
+  | Milestoned    -- ^ The issue was added to a milestone.
+  | Demilestoned  -- ^ The issue was removed from a milestone.
+  | Renamed       -- ^ The issue title was changed.
+  | Locked        -- ^ The issue was locked by the actor.
+  | Unlocked      -- ^ The issue was unlocked by the actor.
+  | HeadRefDeleted -- ^ The pull request’s branch was deleted.
+  | HeadRefRestored -- ^ The pull request’s branch was restored.
   deriving (Show, Data, Typeable, Eq, Ord)
 
 data Event = Event {
@@ -351,9 +369,29 @@ data DetailedPullRequest = DetailedPullRequest {
   ,detailedPullRequestBase :: PullRequestCommit
   ,detailedPullRequestCommits :: Int
   ,detailedPullRequestMerged :: Bool
-  ,detailedPullRequestMergeable :: Bool
+  ,detailedPullRequestMergeable :: Maybe Bool
 } deriving (Show, Data, Typeable, Eq, Ord)
 
+data EditPullRequest = EditPullRequest {
+   editPullRequestTitle :: Maybe String
+  ,editPullRequestBody :: Maybe String
+  ,editPullRequestState :: Maybe EditPullRequestState
+} deriving (Show)
+
+data CreatePullRequest =
+      CreatePullRequest
+      { createPullRequestTitle :: String
+      , createPullRequestBody  :: String
+      , createPullRequestHead  :: String
+      , createPullRequestBase  :: String
+      }
+    | CreatePullRequestIssue
+      { createPullRequestIssueNum :: Int
+      , createPullRequestHead     :: String
+      , createPullRequestBase     :: String
+      }
+    deriving (Show)
+
 data PullRequestLinks = PullRequestLinks {
    pullRequestLinksReviewComments :: String
   ,pullRequestLinksComments :: String
@@ -362,28 +400,33 @@ data PullRequestLinks = PullRequestLinks {
 } deriving (Show, Data, Typeable, Eq, Ord)
 
 data PullRequestCommit = PullRequestCommit {
+   pullRequestCommitLabel :: String
+  ,pullRequestCommitRef :: String
+  ,pullRequestCommitSha :: String
+  ,pullRequestCommitUser :: GithubOwner
+  ,pullRequestCommitRepo :: Repo
 } deriving (Show, Data, Typeable, Eq, Ord)
 
 data SearchReposResult = SearchReposResult {
   searchReposTotalCount :: Int
-  ,searchReposRepos :: [ Repo ]
+  ,searchReposRepos :: [Repo]
 } deriving (Show, Data, Typeable, Eq, Ord)
 
 data Repo = Repo {
-   repoSshUrl :: String
+   repoSshUrl :: Maybe String
   ,repoDescription :: Maybe String
-  ,repoCreatedAt :: GithubDate
+  ,repoCreatedAt :: Maybe GithubDate
   ,repoHtmlUrl :: String
-  ,repoSvnUrl :: String
-  ,repoForks :: Int
+  ,repoSvnUrl :: Maybe String
+  ,repoForks :: Maybe Int
   ,repoHomepage :: Maybe String
-  ,repoFork :: Bool
-  ,repoGitUrl :: String
+  ,repoFork :: Maybe Bool
+  ,repoGitUrl :: Maybe String
   ,repoPrivate :: Bool
-  ,repoCloneUrl :: String
-  ,repoSize :: Int
-  ,repoUpdatedAt :: GithubDate
-  ,repoWatchers :: Int
+  ,repoCloneUrl :: Maybe String
+  ,repoSize :: Maybe Int
+  ,repoUpdatedAt :: Maybe GithubDate
+  ,repoWatchers :: Maybe Int
   ,repoOwner :: GithubOwner
   ,repoName :: String
   ,repoLanguage :: Maybe String
@@ -391,17 +434,49 @@ data Repo = Repo {
   ,repoPushedAt :: Maybe GithubDate   -- ^ this is Nothing for new repositories
   ,repoId :: Int
   ,repoUrl :: String
-  ,repoOpenIssues :: Int
+  ,repoOpenIssues :: Maybe Int
   ,repoHasWiki :: Maybe Bool
   ,repoHasIssues :: Maybe Bool
   ,repoHasDownloads :: Maybe Bool
   ,repoParent :: Maybe RepoRef
   ,repoSource :: Maybe RepoRef
+  ,repoHooksUrl :: String
 } deriving (Show, Data, Typeable, Eq, Ord)
 
 data RepoRef = RepoRef GithubOwner String -- Repo owner and name
  deriving (Show, Data, Typeable, Eq, Ord)
 
+data SearchCodeResult = SearchCodeResult {
+  searchCodeTotalCount :: Int
+  ,searchCodeCodes :: [Code]
+} deriving (Show, Data, Typeable, Eq, Ord)
+
+data Code = Code {
+   codeName :: String
+  ,codePath :: String
+  ,codeSha :: String
+  ,codeUrl :: String
+  ,codeGitUrl :: String
+  ,codeHtmlUrl :: String
+  ,codeRepo :: Repo
+} deriving (Show, Data, Typeable, Eq, Ord)
+
+data Content = ContentFile ContentData | ContentDirectory [ContentData]
+ deriving (Show, Data, Typeable, Eq, Ord)
+
+data ContentData = ContentData {
+   contentType :: String
+  ,contentEncoding :: String
+  ,contentSize :: Int
+  ,contentName :: String
+  ,contentPath :: String
+  ,contentData :: String
+  ,contentSha :: String
+  ,contentUrl :: String
+  ,contentGitUrl :: String
+  ,contentHtmlUrl :: String
+} deriving (Show, Data, Typeable, Eq, Ord)
+
 data Contributor
   -- | An existing Github user, with their number of contributions, avatar
   -- URL, login, URL, ID, and Gravatar ID.
@@ -443,7 +518,7 @@ data DetailedOwner = DetailedUser {
   ,detailedOwnerAvatarUrl :: String
   ,detailedOwnerFollowers :: Int
   ,detailedOwnerFollowing :: Int
-  ,detailedOwnerHireable :: Bool
+  ,detailedOwnerHireable :: Maybe Bool
   ,detailedOwnerGravatarId :: Maybe String
   ,detailedOwnerBlog :: Maybe String
   ,detailedOwnerBio :: Maybe String
@@ -451,7 +526,7 @@ data DetailedOwner = DetailedUser {
   ,detailedOwnerName :: Maybe String
   ,detailedOwnerLocation :: Maybe String
   ,detailedOwnerCompany :: Maybe String
-  ,detailedOwnerEmail :: String
+  ,detailedOwnerEmail :: Maybe String
   ,detailedOwnerUrl :: String
   ,detailedOwnerId :: Int
   ,detailedOwnerHtmlUrl :: String
@@ -475,3 +550,75 @@ data DetailedOwner = DetailedUser {
   ,detailedOwnerHtmlUrl :: String
   ,detailedOwnerLogin :: String
 } deriving (Show, Data, Typeable, Eq, Ord)
+
+data RepoWebhook = RepoWebhook {
+   repoWebhookUrl :: String
+  ,repoWebhookTestUrl :: String
+  ,repoWebhookId :: Integer
+  ,repoWebhookName :: String
+  ,repoWebhookActive :: Bool
+  ,repoWebhookEvents :: [RepoWebhookEvent]
+  ,repoWebhookConfig :: M.Map String String
+  ,repoWebhookLastResponse :: RepoWebhookResponse
+  ,repoWebhookUpdatedAt :: GithubDate
+  ,repoWebhookCreatedAt :: GithubDate
+} deriving (Show, Data, Typeable, Eq, Ord)
+
+data RepoWebhookEvent =
+   WebhookWildcardEvent
+ | WebhookCommitCommentEvent
+ | WebhookCreateEvent
+ | WebhookDeleteEvent
+ | WebhookDeploymentEvent
+ | WebhookDeploymentStatusEvent
+ | WebhookForkEvent
+ | WebhookGollumEvent
+ | WebhookIssueCommentEvent
+ | WebhookIssuesEvent
+ | WebhookMemberEvent
+ | WebhookPageBuildEvent
+ | WebhookPublicEvent
+ | WebhookPullRequestReviewCommentEvent
+ | WebhookPullRequestEvent
+ | WebhookPushEvent
+ | WebhookReleaseEvent
+ | WebhookStatusEvent
+ | WebhookTeamAddEvent
+ | WebhookWatchEvent
+   deriving (Show, Data, Typeable, Eq, Ord)
+
+data RepoWebhookResponse = RepoWebhookResponse {
+   repoWebhookResponseCode :: Maybe Int
+  ,repoWebhookResponseStatus :: String
+  ,repoWebhookResponseMessage :: Maybe String
+} deriving (Show, Data, Typeable, Eq, Ord)
+
+data PullRequestEvent = PullRequestEvent {
+   pullRequestEventAction :: PullRequestEventType
+  ,pullRequestEventNumber :: Int
+  ,pullRequestEventPullRequest :: DetailedPullRequest
+  ,pullRequestRepository :: Repo
+  ,pullRequestSender :: GithubOwner
+} deriving (Show, Data, Typeable, Eq, Ord)
+
+data PullRequestEventType =
+    PullRequestOpened
+  | PullRequestClosed
+  | PullRequestSynchronized
+  | PullRequestReopened
+  | PullRequestAssigned
+  | PullRequestUnassigned
+  | PullRequestLabeled
+  | PullRequestUnlabeled
+  deriving (Show, Data, Typeable, Eq, Ord)
+
+data PingEvent = PingEvent {
+   pingEventZen :: String
+  ,pingEventHook :: RepoWebhook
+  ,pingEventHookId :: Int
+} deriving (Show, Data, Typeable, Eq, Ord)
+
+data EditPullRequestState =
+    EditPullRequestStateOpen
+  | EditPullRequestStateClosed
+  deriving Show
