@@ -14,13 +14,14 @@ formatRepo repo =
   (Github.repoName repo) ++ "\t" ++
     (fromMaybe "" $ Github.repoDescription repo) ++ "\n" ++
     (Github.repoHtmlUrl repo) ++ "\n" ++
-    (Github.repoCloneUrl repo) ++ "\t" ++
+    (fromMaybe "" $ Github.repoCloneUrl repo) ++ "\t" ++
     (formatDate $ Github.repoUpdatedAt repo) ++ "\n" ++
     formatLanguage (Github.repoLanguage repo) ++
     "watchers: " ++ (show $ Github.repoWatchers repo) ++ "\t" ++
     "forks: " ++ (show $ Github.repoForks repo)
 
-formatDate = show . Github.fromGithubDate
+formatDate (Just date) = show . Github.fromGithubDate $ date
+formatDate Nothing = "????"
 
 formatLanguage (Just language) = "language: " ++ language ++ "\t"
 formatLanguage Nothing = ""
